/*
 * AIEngine a new generation network intrusion detection system.
 *
 * Copyright (C) 2013-2020  Luis Campo Giralte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 
 *
 */
#include "GREProtocol.h"
#include <iomanip>

namespace aiengine {

GREProtocol::GREProtocol():
	Protocol("GREProtocol", "gre") {}

bool GREProtocol::greChecker(Packet &packet) {

	int length = packet.getLength();

	if (length >= header_size) {
		setHeader(packet.getPayload());

		if (getProtocol() == ETH_P_TEB) {
			// We just accept packets that are full transparent
			++total_valid_packets_;
			return true;
		}
	}
	++total_invalid_packets_;
	return false;
}

bool GREProtocol::processPacket(Packet &packet) {

        ++total_packets_;
        total_bytes_ += packet.getLength();

	if (!mux_.expired()) {
        	MultiplexerPtr mux = mux_.lock();
     
                mux->setNextProtocolIdentifier(0);

                mux->setHeaderSize(header_size);
                packet.setPrevHeaderSize(header_size);
        }

	return true;
}

void GREProtocol::statistics(std::basic_ostream<char> &out, int level) { 

	showStatisticsHeader(out, level);

	if (level > 2) {
		if (mux_.lock())
			mux_.lock()->statistics(out);
	}
}

void GREProtocol::statistics(nlohmann::json &out, int level) {          

	showStatisticsHeader(out, level);
}

CounterMap GREProtocol::getCounters() const {
 	CounterMap cm; 

        cm.addKeyValue("packets", total_packets_);
        cm.addKeyValue("bytes", total_bytes_);

        return cm;
}

} // namespace aiengine
